@@ -141,25 +141,60 @@ class ScopeProviderTest(UnitTest):
             """
         )
         scope_of_module = scopes[m]
-        for idx, in_scope in enumerate(["foo", "fizzbuzz", "a", "g"]):
-            self.assertEqual(
-                len(scope_of_module[in_scope]), 1, f"{in_scope} should be in scope."
-            )
+        for idx, in_scopes in enumerate(
+            [["foo", "foo.bar"], ["fizzbuzz"], ["a", "a.b", "a.b.c"], ["g"],]
+        ):
+            for in_scope in in_scopes:
+                self.assertEqual(
+                    len(scope_of_module[in_scope]), 1, f"{in_scope} should be in scope."
+                )
 
-            assignment = cast(Assignment, list(scope_of_module[in_scope])[0])
-            self.assertEqual(
-                assignment.name,
-                in_scope,
-                f"Assignment name {assignment.name} should equal to {in_scope}.",
-            )
-            import_node = ensure_type(m.body[idx], cst.SimpleStatementLine).body[0]
-            self.assertEqual(
-                assignment.node,
-                import_node,
-                f"The node of Assignment {assignment.node} should equal to {import_node}",
-            )
+                assignment = cast(Assignment, list(scope_of_module[in_scope])[0])
+                self.assertEqual(
+                    assignment.name,
+                    in_scope,
+                    f"Assignment name {assignment.name} should equal to {in_scope}.",
+                )
+                import_node = ensure_type(m.body[idx], cst.SimpleStatementLine).body[0]
+                self.assertEqual(
+                    assignment.node,
+                    import_node,
+                    f"The node of Assignment {assignment.node} should equal to {import_node}",
+                )
 
-    def test_imoprt_from(self) -> None:
+    def test_dotted_import_access(self) -> None:
+        m, scopes = get_scope_metadata_provider(
+            """
+            import a.b.c, x.y
+            a.b.c(x.z)
+            """
+        )
+        scope_of_module = scopes[m]
+        first_statement = ensure_type(m.body[1], cst.SimpleStatementLine)
+        call = ensure_type(
+            ensure_type(first_statement.body[0], cst.Expr).value, cst.Call
+        )
+        self.assertTrue("a.b.c" in scope_of_module)
+        self.assertTrue("a" in scope_of_module)
+        self.assertEqual(scope_of_module.accesses["a"], set())
+
+        a_b_c_assignment = cast(Assignment, list(scope_of_module["a.b.c"])[0])
+        a_b_c_access = list(a_b_c_assignment.references)[0]
+        self.assertEqual(scope_of_module.accesses["a.b.c"], {a_b_c_access})
+        self.assertEqual(a_b_c_access.node, call.func)
+
+        x_assignment = cast(Assignment, list(scope_of_module["x"])[0])
+        x_access = list(x_assignment.references)[0]
+        self.assertEqual(scope_of_module.accesses["x"], {x_access})
+        self.assertEqual(
+            x_access.node, ensure_type(call.args[0].value, cst.Attribute).value
+        )
+
+        self.assertTrue("x.y" in scope_of_module)
+        self.assertEqual(list(scope_of_module["x.y"])[0].references, set())
+        self.assertEqual(scope_of_module.accesses["x.y"], set())
+
+    def test_import_from(self) -> None:
         m, scopes = get_scope_metadata_provider(
             """
             from foo.bar import a, b as b_renamed
@@ -752,6 +787,48 @@ class ScopeProviderTest(UnitTest):
             {QualifiedName("f4.<locals>.f5.<locals>.C", QualifiedNameSource.LOCAL)},
         )
 
+    def test_get_qualified_names_for_dotted_imports(self) -> None:
+        m, scopes = get_scope_metadata_provider(
+            """
+                import a.b.c
+                a(a.b.d)
+            """
+        )
+        call = ensure_type(
+            ensure_type(
+                ensure_type(m.body[1], cst.SimpleStatementLine).body[0], cst.Expr
+            ).value,
+            cst.Call,
+        )
+        module_scope = scopes[m]
+        self.assertEqual(
+            module_scope.get_qualified_names_for(call.func),
+            {QualifiedName("a", QualifiedNameSource.IMPORT)},
+        )
+        self.assertEqual(
+            module_scope.get_qualified_names_for(call.args[0].value),
+            {QualifiedName("a.b.d", QualifiedNameSource.IMPORT)},
+        )
+
+        import_stmt = ensure_type(
+            ensure_type(m.body[0], cst.SimpleStatementLine).body[0], cst.Import
+        )
+        a_b_c = ensure_type(import_stmt.names[0].name, cst.Attribute)
+        a_b = ensure_type(a_b_c.value, cst.Attribute)
+        a = a_b.value
+        self.assertEqual(
+            module_scope.get_qualified_names_for(a_b_c),
+            {QualifiedName("a.b.c", QualifiedNameSource.IMPORT)},
+        )
+        self.assertEqual(
+            module_scope.get_qualified_names_for(a_b),
+            {QualifiedName("a.b", QualifiedNameSource.IMPORT)},
+        )
+        self.assertEqual(
+            module_scope.get_qualified_names_for(a),
+            {QualifiedName("a", QualifiedNameSource.IMPORT)},
+        )
+
     def test_multiple_assignments(self) -> None:
         m, scopes = get_scope_metadata_provider(
             """
@@ -782,7 +859,7 @@ class ScopeProviderTest(UnitTest):
             },
         )
 
-    def test_assignemnts_and_accesses(self) -> None:
+    def test_assignments_and_accesses(self) -> None:
         m, scopes = get_scope_metadata_provider(
             """
                 a = 1
@@ -1029,6 +1106,18 @@ class ScopeProviderTest(UnitTest):
                     list(scope.parent._accesses.items()), before_parent_accesses
                 )
 
+    def test_attribute_of_function_call(self) -> None:
+        get_scope_metadata_provider("foo().bar")
+
+    def test_attribute_of_subscript_called(self) -> None:
+        m, scopes = get_scope_metadata_provider("foo[0].bar.baz()")
+        scope = scopes[m]
+        self.assertIn("foo", scope.accesses)
+
+    def test_self(self) -> None:
+        with open(__file__) as f:
+            get_scope_metadata_provider(f.read())
+
     def test_get_qualified_names_for_is_read_only(self) -> None:
         m, scopes = get_scope_metadata_provider(
             """
